#include <iostream>
#include <thread>
#include <future>

using namespace std;

void multiply(
        future<int> inputFuture1,
        future<int> inputFuture2,
        promise<int> &outputPromise) {
    int input1 = inputFuture1.get();
    int input2 = inputFuture2.get();
    outputPromise.set_value(input1 * input2);
}

int main() {
    promise<int> inputPromise1;
    promise<int> inputPromise2;
    promise<int> outputPromise;

    // Future objects are "moved", i.e., they are transferred
    // from the main thread to multiply() without copy.
    thread worker(
            multiply,
            move(inputPromise1.get_future()),
            move(inputPromise2.get_future()),
            ref(outputPromise));

    // Set the input values for the worker thread.
    inputPromise1.set_value(10);
    inputPromise2.set_value(5);

    future<int> outputFuture = outputPromise.get_future();
    cout << "Output obtained in main thread: " << outputFuture.get() << endl;

    worker.join();
    return 0;
}
